@@ -1,4 +1,4 @@
-/*  $Id: launcher-exec.c 28579 2008-11-02 10:08:13Z nick $
+/*  $Id: launcher-exec.c 29085 2009-01-02 20:38:36Z nick $
  *
  *  Copyright (c) 2006-2007 Nick Schermer <nick@xfce.org>
  *
@@ -21,6 +21,7 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -28,12 +29,18 @@
 #include <sys/wait.h>
 #endif
 
+#ifdef HAVE_FEATURES_H
+#include <features.h>
+#endif
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
 #ifdef HAVE_MEMORY_H
 #include <memory.h>
 #endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -404,9 +411,9 @@ launcher_exec_on_screen (GdkScreen     *screen,
     SnDisplay            *sn_display = NULL;
     LauncherStartupData  *startup_data;
     gint                  sn_workspace;
-    extern gchar        **environ;
-    gint                  n, m;
 #endif
+    gchar                *dpyname;
+    gint                  n, m;
     gboolean              succeed = FALSE;
     GError               *error = NULL;
     gchar               **argv;
@@ -454,15 +461,21 @@ launcher_exec_on_screen (GdkScreen     *screen,
                     ;
 
                 /* alloc new envp string */
-                envp = g_new (gchar *, n + 2);
+                envp = g_new (gchar *, n + 3);
 
                 /* copy the environ vars into the envp */
                 for (n = m = 0; environ[n] != NULL; ++n)
-                    if (G_LIKELY (strncmp (environ[n], "DESKTOP_STARTUP_ID", 18) != 0))
+                    if (G_LIKELY (strncmp (environ[n], "DESKTOP_STARTUP_ID", 18) != 0 && strncmp (environ[n], "DISPLAY", 7) != 0))
                         envp[m++] = g_strdup (environ[n]);
 
                 /* append the startup notification id */
                 envp[m++] = g_strconcat ("DESKTOP_STARTUP_ID=", sn_launcher_context_get_startup_id (sn_launcher), NULL);
+
+                /* workaround for the failure of gdk_spawn_on_screen to setup the correct DISPLAY env */
+                dpyname = gdk_screen_make_display_name (screen);
+                envp[m++] = g_strconcat ("DISPLAY=", dpyname, NULL);
+                g_free (dpyname);
+
                 envp[m] = NULL;
 
                 /* we want to watch the child process */
@@ -470,6 +483,25 @@ launcher_exec_on_screen (GdkScreen     *screen,
             }
         }
     }
+#else
+    /* count environ items */
+    for (n = 0; environ[n] != NULL; ++n)
+        ;
+
+    /* alloc new envp string */
+    envp = g_new (gchar *, n + 2);
+
+    /* copy the environ vars into the envp */
+    for (n = m = 0; environ[n] != NULL; ++n)
+        if (G_LIKELY (strncmp (environ[n], "DISPLAY", 7) != 0))
+            envp[m++] = g_strdup (environ[n]);
+
+    /* workaround for the failure of gdk_spawn_on_screen to setup the correct DISPLAY env */
+    dpyname = gdk_screen_make_display_name (screen);
+    envp[m++] = g_strconcat ("DISPLAY=", dpyname, NULL);
+    g_free (dpyname);
+
+    envp[m] = NULL;
 #endif
 
     /* spawn the application */
@@ -612,7 +644,7 @@ launcher_execute_from_clipboard (GdkScreen     *screen,
         text = gtk_clipboard_wait_for_text (clipboard);
 
     /* try other clipboard if this one was empty */
-    if (text == NULL)
+    if (text == NULL || *text == '\0')
     {
         /* get the clipboard */
         clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
@@ -622,11 +654,12 @@ launcher_execute_from_clipboard (GdkScreen     *screen,
             text = gtk_clipboard_wait_for_text (clipboard);
     }
 
-    if (G_LIKELY (text))
+    if (G_LIKELY (text != NULL && *text != '\0'))
     {
         /* create some fake selection data */
         selection_data.data = (guchar *) text;
         selection_data.length = strlen (text);
+        selection_data.target = NULL;
 
         /* parse the filelist, this way we can handle 'copied' file from thunar */
         filenames = launcher_utility_filenames_from_selection_data (&selection_data);
