@@ -1,4 +1,4 @@
-/*  $Id: launcher.c 28387 2008-10-23 17:50:09Z nick $
+/*  $Id: launcher.c 29188 2009-01-12 17:08:56Z nick $
  *
  *  Copyright (c) 2005-2007 Jasper Huijsmans <jasper@xfce.org>
  *  Copyright (c) 2006-2007 Nick Schermer <nick@xfce.org>
@@ -66,7 +66,7 @@ static void            launcher_icon_button_drag_data_received      (GtkWidget
                                                                      gint                   y,
                                                                      GtkSelectionData      *selection_data,
                                                                      guint                  info,
-                                                                     guint                  time,
+                                                                     guint                  time_,
                                                                      LauncherPlugin        *launcher);
 static gboolean        launcher_arrow_button_pressed                (GtkWidget             *button,
                                                                      GdkEventButton        *event,
@@ -128,17 +128,17 @@ launcher_utility_filenames_from_selection_data (GtkSelectionData *selection_data
     /* check whether the retrieval worked */
     if (G_LIKELY (selection_data->length > 0))
     {
-        if (g_str_equal(gdk_atom_name(selection_data->target), "text/uri-list"))
+        if (selection_data->target == gdk_atom_intern_static_string ("text/uri-list"))
         {
-			/* split the received uri list */
-			uri_list = g_uri_list_extract_uris ((gchar *) selection_data->data);
+                /* split the received uri list */
+                uri_list = g_uri_list_extract_uris ((gchar *) selection_data->data);
         }
         else
         {
-        	/* split input by \n, \r or \r\n, this might result in empty elements, we sort
-        	 * them out below */
-        	uri_list = g_strsplit_set ((gchar *) selection_data->data, "\n\r", 0);
-        	is_uri = FALSE;
+                /* split input by \n, \r or \r\n, this might result in empty elements, we sort
+                 * them out below */
+                uri_list = g_strsplit_set ((gchar *) selection_data->data, "\n\r", 0);
+                is_uri = FALSE;
         }
 
         if (G_LIKELY (uri_list))
@@ -153,7 +153,7 @@ launcher_utility_filenames_from_selection_data (GtkSelectionData *selection_data
                 if (is_uri)
                     filename = g_filename_from_uri (uri_list[i], NULL, NULL);
                 else
-					filename = g_strdup (uri_list[i]);
+                                        filename = g_strdup (uri_list[i]);
 
                 /* prepend the filename */
                 if (G_LIKELY (filename))
@@ -224,37 +224,33 @@ launcher_utility_query_tooltip (GtkWidget     *widget,
                                 GtkTooltip    *tooltip,
                                 LauncherEntry *entry)
 {
-    gchar     *string;
-    GdkPixbuf *pixbuf;
+    gchar *string;
 
     /* create tooltip text */
     if (G_LIKELY (entry && entry->name))
     {
         if (entry->comment)
+        {
             string = g_strdup_printf ("<b>%s</b>\n%s", entry->name, entry->comment);
+            gtk_tooltip_set_markup (tooltip, string);
+            g_free (string);
+        }
         else
-            string = g_strdup_printf ("%s", entry->name);
-
-        /* set the markup tooltip */
-        gtk_tooltip_set_markup (tooltip, string);
-
-        /* cleanup */
-        g_free (string);
+        {
+            gtk_tooltip_set_text (tooltip, entry->name);
+        }
 
         if (G_LIKELY (entry->icon))
         {
-            /* try to load an pixbuf */
-            pixbuf = launcher_utility_load_pixbuf (gtk_widget_get_screen (widget), entry->icon,
-                                                   LAUNCHER_TOOLTIP_SIZE);
+            /* load the cached pixbuf */
+            if (entry->tooltip_cache == NULL)
+                entry->tooltip_cache = launcher_utility_load_pixbuf (gtk_widget_get_screen (widget),
+                                                                     entry->icon,
+                                                                     LAUNCHER_TOOLTIP_SIZE);
 
-            if (G_LIKELY (pixbuf))
-            {
-                /* set the tooltip icon */
-                gtk_tooltip_set_icon (tooltip, pixbuf);
-
-                /* releases */
-                g_object_unref (G_OBJECT (pixbuf));
-            }
+            /* set the tooltip icon */
+            if (G_LIKELY (entry->tooltip_cache))
+                gtk_tooltip_set_icon (tooltip, entry->tooltip_cache);
         }
 
         /* show the tooltip */
@@ -478,7 +474,7 @@ launcher_icon_button_drag_data_received (GtkWidget        *widget,
                                          gint              y,
                                          GtkSelectionData *selection_data,
                                          guint             info,
-                                         guint             time,
+                                         guint             time_,
                                          LauncherPlugin   *launcher)
 {
     GSList        *filenames;
@@ -504,7 +500,7 @@ launcher_icon_button_drag_data_received (GtkWidget        *widget,
     }
 
     /* finish drag */
-    gtk_drag_finish (context, TRUE, FALSE, time);
+    gtk_drag_finish (context, TRUE, FALSE, time_);
 }
 
 
@@ -624,7 +620,7 @@ launcher_menu_popup (gpointer user_data)
     /* popup menu */
     gtk_menu_popup (GTK_MENU (launcher->menu), NULL, NULL,
                     xfce_panel_plugin_position_menu,
-                    launcher->panel_plugin, 
+                    launcher->panel_plugin,
                     1, gtk_get_current_event_time ());
 
     GDK_THREADS_LEAVE ();
@@ -693,7 +689,7 @@ launcher_menu_rebuild (LauncherPlugin *launcher)
 
     /* set the menu screen */
     gtk_menu_set_screen (GTK_MENU (launcher->menu), screen);
-    
+
     /* get the arrow direction of the button */
     arrow_type = xfce_arrow_button_get_arrow_type (XFCE_ARROW_BUTTON (launcher->arrow_button));
 
@@ -706,10 +702,9 @@ launcher_menu_rebuild (LauncherPlugin *launcher)
 
         entry = li->data;
 
-        /* create menu item */
         mi = gtk_image_menu_item_new_with_label (entry->name ? entry->name : _("New Item"));
         gtk_widget_show (mi);
-        
+
         /* fix menu order when it's on the top or bottom of the screen */
         if (arrow_type == GTK_ARROW_DOWN)
             gtk_menu_shell_append (GTK_MENU_SHELL (launcher->menu), mi);
@@ -740,7 +735,7 @@ launcher_menu_rebuild (LauncherPlugin *launcher)
         /* connect signals */
         g_signal_connect (G_OBJECT (mi), "button-release-event", G_CALLBACK (launcher_menu_item_released), launcher);
 #if LAUNCHER_NEW_TOOLTIP_API
-        g_object_set (G_OBJECT (mi), "has-tooltip", TRUE, NULL);
+        gtk_widget_set_has_tooltip (mi, TRUE);
         g_signal_connect (G_OBJECT (mi), "query-tooltip", G_CALLBACK (launcher_utility_query_tooltip), entry);
 #endif
 
@@ -771,7 +766,7 @@ launcher_entry_new (void)
     /* allocate structure */
     entry = panel_slice_new0 (LauncherEntry);
 
-    /* set some default values */
+    /* TRANSLATORS: Name of a newly created launcher */
     entry->name    = g_strdup (_("New Item"));
     entry->comment = NULL;
     entry->icon    = g_strdup ("applications-other");
@@ -783,6 +778,9 @@ launcher_entry_new (void)
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
     entry->startup  = FALSE;
 #endif
+#if LAUNCHER_NEW_TOOLTIP_API
+    entry->tooltip_cache = NULL;
+#endif
 
     return entry;
 }
@@ -804,6 +802,12 @@ launcher_entry_free (LauncherEntry  *entry,
     g_free (entry->icon);
     g_free (entry->exec);
 
+#if LAUNCHER_NEW_TOOLTIP_API
+    /* release cached tooltip icon */
+    if (entry->tooltip_cache)
+      g_object_unref (G_OBJECT (entry->tooltip_cache));
+#endif
+
     /* free structure */
     panel_slice_free (LauncherEntry, entry);
 }
@@ -856,7 +860,7 @@ launcher_plugin_new (XfcePanelPlugin *plugin)
     /* hook for icon themes changes */
     screen = gtk_widget_get_screen (launcher->image);
     icon_theme = gtk_icon_theme_get_for_screen (screen);
-    g_signal_connect (G_OBJECT (icon_theme), "changed", 
+    g_signal_connect (G_OBJECT (icon_theme), "changed",
                       G_CALLBACK (launcher_utility_icon_theme_changed), launcher);
 
     /* icon button signals */
@@ -872,7 +876,7 @@ launcher_plugin_new (XfcePanelPlugin *plugin)
                             G_CALLBACK (launcher_icon_button_expose_event), launcher);
 
 #if LAUNCHER_NEW_TOOLTIP_API
-    g_object_set (G_OBJECT (launcher->icon_button), "has-tooltip", TRUE, NULL);
+    gtk_widget_set_has_tooltip (launcher->icon_button, TRUE);
     g_signal_connect (G_OBJECT (launcher->icon_button), "query-tooltip",
                       G_CALLBACK (launcher_icon_button_query_tooltip), launcher);
 #endif
@@ -1167,7 +1171,7 @@ launcher_plugin_screen_position_changed (LauncherPlugin *launcher)
 
     /* set the arrow direction */
     xfce_arrow_button_set_arrow_type (XFCE_ARROW_BUTTON (launcher->arrow_button), arrow_type);
-    
+
     /* destroy the menu, so menu items appear in the correct direction */
     launcher_menu_destroy (launcher);
 }
